#include <iostream>
#include <vector>
#include <algorithm>
#include <set>
#include <map>

// C++ 17!!

// STL -- Standard Template Library
//
// Containers: vectors, sets, maps
// Iterators
// Functions
// Algorithms std::sort(my_vector.begin(), my_vector.end());

// sort(int[] a, int size, comparison function)

template <class T>
void PrintVector(std::vector<T> &v) {
  // for (int i = 0; i < v.size(); i++) {
  //     std::cout << "v[" << i << "] = " << v[i] << std::endl;
  // }

    // range-based for loop example
    int index = 0;
    for (T& x : v) {
     std::cout << "v[" << index << "] = " << x << std::endl;
     ++index;
    }
}

void VectorExample() {
  std::vector<int> v;

  for (int i = 0; i < 10; ++i) {
    v.push_back(i * i);
  }

  v.pop_back();
  std::vector<int>::iterator p = std::find(v.begin(), v.end(), 25);
  *p = 6;
  PrintVector(v);
  // std::cout << v.at(40) << std::endl;
}

using std::set;

void SetExample() {
  set<int> s;
  s.insert(100);
  for (int i = 0; i < 5; ++i) {
    s.insert(i * i);
  }
  s.erase(9);
  s.erase(127);

  for (auto& i : s) {
    std::cout << i << ", ";
  }
  std::cout << std::endl;

  int x = 4;
  auto i = s.find(x);
  if (s.count(x) == 0) {
    std::cout << "did not find " << x << " in s\n";
  } else {
    std::cout << "found " << x << " in s\n";
  }
}

void MapExample() {
  std::map<int, std::string> s {
    {0, "Mehmet"},
    {-3, "Alice"},
    {42, "Bob"},
  };

  s.find(42)->second = "Charlie";
  
  for (auto i = s.begin(); i != s.end(); ++i) {
    std::pair<int, std::string> x = *i;
    std::cout << x.first << " : " << x.second << "\n";
  }

  // TODO: range-based for, deconstruction

  s.insert({9, "Dana"});

  auto x = s.find(9);
  if (x == s.end()) {
    s.insert({9, "Ellen"});
  } else {
    x->second = "Ellen";
  }

  s[10] = "Fyodor";
  s[11];
  s.insert_or_assign(10, "Gabe");
  
  auto y = s.find(9);
  if (y != s.end()) {
    std::cout << "found " << y->second << "\n";
  } else {
    std::cout << "did not find\n";
  }

  for (auto i = s.begin(); i != s.end(); ++i) {
    std::pair<int, std::string> x = *i;
    std::cout << x.first << " : " << x.second << "\n";
  }
}

int main() {
  MapExample();
  return 0;
}
